@@ -8,29 +8,34 @@
 #include "QXmppConstants_p.h"
 #include "QXmppJingleIq.h"
 #include "QXmppUtils.h"
+#include "QXmppUtils_p.h"
+
+#include "StringLiterals.h"
 
 #include <QDateTime>
 #include <QDomElement>
 
-static const QStringList PRESENCE_TYPES = {
-    QStringLiteral("error"),
-    QString(),
-    QStringLiteral("unavailable"),
-    QStringLiteral("subscribe"),
-    QStringLiteral("subscribed"),
-    QStringLiteral("unsubscribe"),
-    QStringLiteral("unsubscribed"),
-    QStringLiteral("probe")
-};
+using namespace QXmpp::Private;
 
-static const QStringList AVAILABLE_STATUS_TYPES = {
-    QString(),
-    QStringLiteral("away"),
-    QStringLiteral("xa"),
-    QStringLiteral("dnd"),
-    QStringLiteral("chat"),
-    QStringLiteral("invisible")
-};
+constexpr auto PRESENCE_TYPES = to_array<QStringView>({
+    u"error",
+    {},
+    u"unavailable",
+    u"subscribe",
+    u"subscribed",
+    u"unsubscribe",
+    u"unsubscribed",
+    u"probe",
+});
+
+constexpr auto AVAILABLE_STATUS_TYPES = to_array<QStringView>({
+    {},
+    u"away",
+    u"xa",
+    u"dnd",
+    u"chat",
+    u"invisible",
+});
 
 class QXmppPresencePrivate : public QSharedData
 {
@@ -113,105 +118,93 @@ bool QXmppPresence::isXmppStanza() const
     return true;
 }
 
+///
 /// Returns the availability status type, for instance busy or away.
 ///
 /// This will not tell you whether a contact is connected, check whether
 /// type() is QXmppPresence::Available instead.
-
+///
 QXmppPresence::AvailableStatusType QXmppPresence::availableStatusType() const
 {
     return d->availableStatusType;
 }
 
 /// Sets the availability status type, for instance busy or away.
-
 void QXmppPresence::setAvailableStatusType(AvailableStatusType type)
 {
     d->availableStatusType = type;
 }
 
 /// Returns the priority level of the resource.
-
 int QXmppPresence::priority() const
 {
     return d->priority;
 }
 
 /// Sets the \a priority level of the resource.
-
 void QXmppPresence::setPriority(int priority)
 {
     d->priority = priority;
 }
 
 /// Returns the status text, a textual description of the user's status.
-
 QString QXmppPresence::statusText() const
 {
     return d->statusText;
 }
 
+///
 /// Sets the status text, a textual description of the user's status.
 ///
 /// \param statusText The status text, for example "Gone fishing".
-
+///
 void QXmppPresence::setStatusText(const QString &statusText)
 {
     d->statusText = statusText;
 }
 
+///
 /// Returns the presence type.
 ///
 /// You can use this method to determine the action which needs to be
 /// taken in response to receiving the presence. For instance, if the type is
 /// QXmppPresence::Available or QXmppPresence::Unavailable, you could update
 /// the icon representing a contact's availability.
-
+///
 QXmppPresence::Type QXmppPresence::type() const
 {
     return d->type;
 }
 
 /// Sets the presence type.
-///
-/// \param type
-
 void QXmppPresence::setType(QXmppPresence::Type type)
 {
     d->type = type;
 }
 
 /// Returns the photo-hash of the VCardUpdate.
-///
-/// \return QByteArray
-
 QByteArray QXmppPresence::photoHash() const
 {
     return d->photoHash;
 }
 
+///
 /// Sets the photo-hash of the VCardUpdate.
 ///
 /// \param photoHash as QByteArray
-
+///
 void QXmppPresence::setPhotoHash(const QByteArray &photoHash)
 {
     d->photoHash = photoHash;
 }
 
 /// Returns the type of VCardUpdate
-///
-/// \return VCardUpdateType
-
 QXmppPresence::VCardUpdateType QXmppPresence::vCardUpdateType() const
 {
     return d->vCardUpdateType;
 }
 
 /// Sets the type of VCardUpdate
-///
-/// \param type VCardUpdateType
-
 void QXmppPresence::setVCardUpdateType(VCardUpdateType type)
 {
     d->vCardUpdateType = type;
@@ -308,60 +301,48 @@ void QXmppPresence::setMujiContents(const QVector<QXmppJingleIq::Content> &mujiC
 }
 
 /// Returns the MUC item.
-
 QXmppMucItem QXmppPresence::mucItem() const
 {
     return d->mucItem;
 }
 
 /// Sets the MUC item.
-///
-/// \param item
-
 void QXmppPresence::setMucItem(const QXmppMucItem &item)
 {
     d->mucItem = item;
 }
 
 /// Returns the password used to join a MUC room.
-
 QString QXmppPresence::mucPassword() const
 {
     return d->mucPassword;
 }
 
 /// Sets the password used to join a MUC room.
-
 void QXmppPresence::setMucPassword(const QString &password)
 {
     d->mucPassword = password;
 }
 
 /// Returns the MUC status codes.
-
 QList<int> QXmppPresence::mucStatusCodes() const
 {
     return d->mucStatusCodes;
 }
 
 /// Sets the MUC status codes.
-///
-/// \param codes
-
 void QXmppPresence::setMucStatusCodes(const QList<int> &codes)
 {
     d->mucStatusCodes = codes;
 }
 
 /// Returns true if the sender has indicated MUC support.
-
 bool QXmppPresence::isMucSupported() const
 {
     return d->mucSupported;
 }
 
 /// Sets whether MUC is \a supported.
-
 void QXmppPresence::setMucSupported(bool supported)
 {
     d->mucSupported = supported;
@@ -389,37 +370,41 @@ void QXmppPresence::setLastUserInteraction(const QDateTime &lastUserInteraction)
     d->lastUserInteraction = lastUserInteraction;
 }
 
+///
 /// Returns the actual (full) JID of the MIX channel participant.
 ///
 /// \since QXmpp 1.1
-
+///
 QString QXmppPresence::mixUserJid() const
 {
     return d->mixUserJid;
 }
 
+///
 /// Sets the actual (full) JID of the MIX channel participant.
 ///
 /// \since QXmpp 1.1
-
+///
 void QXmppPresence::setMixUserJid(const QString &mixUserJid)
 {
     d->mixUserJid = mixUserJid;
 }
 
+///
 /// Returns the MIX participant's nickname.
 ///
 /// \since QXmpp 1.1
-
+///
 QString QXmppPresence::mixUserNick() const
 {
     return d->mixUserNick;
 }
 
+///
 /// Sets the MIX participant's nickname.
 ///
 /// \since QXmpp 1.1
-
+///
 void QXmppPresence::setMixUserNick(const QString &mixUserNick)
 {
     d->mixUserNick = mixUserNick;
@@ -431,31 +416,24 @@ void QXmppPresence::parse(const QDomElement &element)
     QXmppStanza::parse(element);
 
     // attributes
-    int type = PRESENCE_TYPES.indexOf(element.attribute(QStringLiteral("type")));
-    if (type > -1) {
-        d->type = Type(type);
-    }
+    d->type = enumFromString<Type>(PRESENCE_TYPES, element.attribute(u"type"_s))
+                  .value_or(Available);
 
     QXmppElementList unknownElements;
-    QDomElement childElement = element.firstChildElement();
-
-    while (!childElement.isNull()) {
-        if (childElement.tagName() == QStringLiteral("show")) {
-            int availableStatusType = AVAILABLE_STATUS_TYPES.indexOf(childElement.text());
-            if (availableStatusType > -1) {
-                d->availableStatusType = AvailableStatusType(availableStatusType);
-            }
-        } else if (childElement.tagName() == QStringLiteral("status")) {
+    for (const auto &childElement : iterChildElements(element)) {
+        if (childElement.tagName() == u"show") {
+            d->availableStatusType = enumFromString<AvailableStatusType>(AVAILABLE_STATUS_TYPES, childElement.text())
+                                         .value_or(Online);
+        } else if (childElement.tagName() == u"status") {
             d->statusText = childElement.text();
-        } else if (childElement.tagName() == QStringLiteral("priority")) {
+        } else if (childElement.tagName() == u"priority") {
             d->priority = childElement.text().toInt();
             // parse presence extensions
             // XEP-0033: Extended Stanza Addressing and errors are parsed by QXmppStanza
-        } else if (!(childElement.tagName() == QStringLiteral("addresses") && childElement.namespaceURI() == ns_extended_addressing) &&
-                   childElement.tagName() != "error") {
+        } else if (!(childElement.tagName() == u"addresses" && childElement.namespaceURI() == ns_extended_addressing) &&
+                   childElement.tagName() != u"error") {
             parseExtension(childElement, unknownElements);
         }
-        childElement = childElement.nextSiblingElement();
     }
 
     setExtensions(unknownElements);
@@ -464,28 +442,25 @@ void QXmppPresence::parse(const QDomElement &element)
 void QXmppPresence::parseExtension(const QDomElement &element, QXmppElementList &unknownElements)
 {
     // XEP-0045: Multi-User Chat
-    if (element.tagName() == QStringLiteral("x") && element.namespaceURI() == ns_muc) {
+    if (element.tagName() == u"x" && element.namespaceURI() == ns_muc) {
         d->mucSupported = true;
-        d->mucPassword = element.firstChildElement(QStringLiteral("password")).text();
-    } else if (element.tagName() == QStringLiteral("x") && element.namespaceURI() == ns_muc_user) {
-        QDomElement itemElement = element.firstChildElement(QStringLiteral("item"));
-        d->mucItem.parse(itemElement);
-        QDomElement statusElement = element.firstChildElement(QStringLiteral("status"));
-        d->mucStatusCodes.clear();
+        d->mucPassword = element.firstChildElement(u"password"_s).text();
+    } else if (element.tagName() == u"x" && element.namespaceURI() == ns_muc_user) {
+        d->mucItem.parse(firstChildElement(element, u"item"));
 
-        while (!statusElement.isNull()) {
-            d->mucStatusCodes << statusElement.attribute(QStringLiteral("code")).toInt();
-            statusElement = statusElement.nextSiblingElement(QStringLiteral("status"));
+        d->mucStatusCodes.clear();
+        for (const auto &statusElement : iterChildElements(element, u"status")) {
+            d->mucStatusCodes << statusElement.attribute(u"code"_s).toInt();
         }
         // XEP-0115: Entity Capabilities
-    } else if (element.tagName() == QStringLiteral("c") && element.namespaceURI() == ns_capabilities) {
-        d->capabilityNode = element.attribute(QStringLiteral("node"));
-        d->capabilityVer = QByteArray::fromBase64(element.attribute(QStringLiteral("ver")).toLatin1());
-        d->capabilityHash = element.attribute(QStringLiteral("hash"));
-        d->capabilityExt = element.attribute(QStringLiteral("ext")).split(' ', Qt::SkipEmptyParts);
+    } else if (element.tagName() == u"c" && element.namespaceURI() == ns_capabilities) {
+        d->capabilityNode = element.attribute(u"node"_s);
+        d->capabilityVer = QByteArray::fromBase64(element.attribute(u"ver"_s).toLatin1());
+        d->capabilityHash = element.attribute(u"hash"_s);
+        d->capabilityExt = element.attribute(u"ext"_s).split(u' ', Qt::SkipEmptyParts);
         // XEP-0153: vCard-Based Avatars
     } else if (element.namespaceURI() == ns_vcard_update) {
-        QDomElement photoElement = element.firstChildElement(QStringLiteral("photo"));
+        QDomElement photoElement = element.firstChildElement(u"photo"_s);
         if (photoElement.isNull()) {
             d->photoHash = {};
             d->vCardUpdateType = VCardUpdateNotReady;
@@ -498,28 +473,26 @@ void QXmppPresence::parseExtension(const QDomElement &element, QXmppElementList
             }
         }
         // XEP-0272: Multiparty Jingle (Muji)
-    } else if (element.tagName() == QStringLiteral("muji") && element.namespaceURI() == ns_muji) {
-        if (!element.firstChildElement(QStringLiteral("preparing")).isNull()) {
+    } else if (element.tagName() == u"muji" && element.namespaceURI() == ns_muji) {
+        if (!element.firstChildElement(u"preparing"_s).isNull()) {
             d->isPreparingMujiSession = true;
         }
 
-        for (auto contentElement = element.firstChildElement(QStringLiteral("content"));
-             !contentElement.isNull();
-             contentElement = contentElement.nextSiblingElement(QStringLiteral("content"))) {
+        for (const auto &contentElement : iterChildElements(element, u"content")) {
             QXmppJingleIq::Content content;
             content.parse(contentElement);
             d->mujiContents.append(content);
         }
         // XEP-0319: Last User Interaction in Presence
-    } else if (element.tagName() == QStringLiteral("idle") && element.namespaceURI() == ns_idle) {
-        if (element.hasAttribute(QStringLiteral("since"))) {
-            const QString since = element.attribute(QStringLiteral("since"));
+    } else if (element.tagName() == u"idle" && element.namespaceURI() == ns_idle) {
+        if (element.hasAttribute(u"since"_s)) {
+            const QString since = element.attribute(u"since"_s);
             d->lastUserInteraction = QXmppUtils::datetimeFromString(since);
         }
         // XEP-0405: Mediated Information eXchange (MIX): Participant Server Requirements
-    } else if (element.tagName() == QStringLiteral("mix") && element.namespaceURI() == ns_mix_presence) {
-        d->mixUserJid = element.firstChildElement(QStringLiteral("jid")).text();
-        d->mixUserNick = element.firstChildElement(QStringLiteral("nick")).text();
+    } else if (element.tagName() == u"mix" && element.namespaceURI() == ns_mix_presence) {
+        d->mixUserJid = element.firstChildElement(u"jid"_s).text();
+        d->mixUserNick = element.firstChildElement(u"nick"_s).text();
     } else {
         unknownElements << element;
     }
@@ -527,45 +500,40 @@ void QXmppPresence::parseExtension(const QDomElement &element, QXmppElementList
 
 void QXmppPresence::toXml(QXmlStreamWriter *xmlWriter) const
 {
-    xmlWriter->writeStartElement(QStringLiteral("presence"));
-    helperToXmlAddAttribute(xmlWriter, QStringLiteral("xml:lang"), lang());
-    helperToXmlAddAttribute(xmlWriter, QStringLiteral("id"), id());
-    helperToXmlAddAttribute(xmlWriter, QStringLiteral("to"), to());
-    helperToXmlAddAttribute(xmlWriter, QStringLiteral("from"), from());
-    helperToXmlAddAttribute(xmlWriter, QStringLiteral("type"), PRESENCE_TYPES.at(d->type));
+    xmlWriter->writeStartElement(QSL65("presence"));
+    writeOptionalXmlAttribute(xmlWriter, u"xml:lang", lang());
+    writeOptionalXmlAttribute(xmlWriter, u"id", id());
+    writeOptionalXmlAttribute(xmlWriter, u"to", to());
+    writeOptionalXmlAttribute(xmlWriter, u"from", from());
+    writeOptionalXmlAttribute(xmlWriter, u"type", PRESENCE_TYPES.at(d->type));
 
-    const QString show = AVAILABLE_STATUS_TYPES.at(d->availableStatusType);
-    if (!show.isEmpty()) {
-        helperToXmlAddTextElement(xmlWriter, QStringLiteral("show"), show);
-    }
-    if (!d->statusText.isEmpty()) {
-        helperToXmlAddTextElement(xmlWriter, QStringLiteral("status"), d->statusText);
-    }
+    writeOptionalXmlTextElement(xmlWriter, u"show", AVAILABLE_STATUS_TYPES.at(size_t(d->availableStatusType)));
+    writeOptionalXmlTextElement(xmlWriter, u"status", d->statusText);
     if (d->priority != 0) {
-        helperToXmlAddTextElement(xmlWriter, QStringLiteral("priority"), QString::number(d->priority));
+        writeXmlTextElement(xmlWriter, u"priority", QString::number(d->priority));
     }
 
     error().toXml(xmlWriter);
 
     // XEP-0045: Multi-User Chat
     if (d->mucSupported) {
-        xmlWriter->writeStartElement(QStringLiteral("x"));
-        xmlWriter->writeDefaultNamespace(ns_muc);
+        xmlWriter->writeStartElement(QSL65("x"));
+        xmlWriter->writeDefaultNamespace(toString65(ns_muc));
         if (!d->mucPassword.isEmpty()) {
-            xmlWriter->writeTextElement(QStringLiteral("password"), d->mucPassword);
+            xmlWriter->writeTextElement(QSL65("password"), d->mucPassword);
         }
         xmlWriter->writeEndElement();
     }
 
     if (!d->mucItem.isNull() || !d->mucStatusCodes.isEmpty()) {
-        xmlWriter->writeStartElement(QStringLiteral("x"));
-        xmlWriter->writeDefaultNamespace(ns_muc_user);
+        xmlWriter->writeStartElement(QSL65("x"));
+        xmlWriter->writeDefaultNamespace(toString65(ns_muc_user));
         if (!d->mucItem.isNull()) {
             d->mucItem.toXml(xmlWriter);
         }
         for (const auto code : d->mucStatusCodes) {
-            xmlWriter->writeStartElement(QStringLiteral("status"));
-            xmlWriter->writeAttribute(QStringLiteral("code"), QString::number(code));
+            xmlWriter->writeStartElement(QSL65("status"));
+            xmlWriter->writeAttribute(QSL65("code"), QString::number(code));
             xmlWriter->writeEndElement();
         }
         xmlWriter->writeEndElement();
@@ -575,24 +543,24 @@ void QXmppPresence::toXml(QXmlStreamWriter *xmlWriter) const
     if (!d->capabilityNode.isEmpty() &&
         !d->capabilityVer.isEmpty() &&
         !d->capabilityHash.isEmpty()) {
-        xmlWriter->writeStartElement(QStringLiteral("c"));
-        xmlWriter->writeDefaultNamespace(ns_capabilities);
-        helperToXmlAddAttribute(xmlWriter, QStringLiteral("hash"), d->capabilityHash);
-        helperToXmlAddAttribute(xmlWriter, QStringLiteral("node"), d->capabilityNode);
-        helperToXmlAddAttribute(xmlWriter, QStringLiteral("ver"), d->capabilityVer.toBase64());
+        xmlWriter->writeStartElement(QSL65("c"));
+        xmlWriter->writeDefaultNamespace(toString65(ns_capabilities));
+        writeOptionalXmlAttribute(xmlWriter, u"hash", d->capabilityHash);
+        writeOptionalXmlAttribute(xmlWriter, u"node", d->capabilityNode);
+        writeOptionalXmlAttribute(xmlWriter, u"ver", QString::fromUtf8(d->capabilityVer.toBase64()));
         xmlWriter->writeEndElement();
     }
 
     // XEP-0153: vCard-Based Avatars
     if (d->vCardUpdateType != VCardUpdateNone) {
-        xmlWriter->writeStartElement(QStringLiteral("x"));
-        xmlWriter->writeDefaultNamespace(ns_vcard_update);
+        xmlWriter->writeStartElement(QSL65("x"));
+        xmlWriter->writeDefaultNamespace(toString65(ns_vcard_update));
         switch (d->vCardUpdateType) {
         case VCardUpdateNoPhoto:
-            xmlWriter->writeEmptyElement(QStringLiteral("photo"));
+            xmlWriter->writeEmptyElement(u"photo"_s);
             break;
         case VCardUpdateValidPhoto:
-            helperToXmlAddTextElement(xmlWriter, QStringLiteral("photo"), d->photoHash.toHex());
+            writeXmlTextElement(xmlWriter, u"photo", QString::fromUtf8(d->photoHash.toHex()));
             break;
         default:
             break;
@@ -602,11 +570,11 @@ void QXmppPresence::toXml(QXmlStreamWriter *xmlWriter) const
 
     // XEP-0272: Multiparty Jingle (Muji)
     if (d->isPreparingMujiSession || !d->mujiContents.isEmpty()) {
-        xmlWriter->writeStartElement(QStringLiteral("muji"));
-        xmlWriter->writeDefaultNamespace(ns_muji);
+        xmlWriter->writeStartElement(QSL65("muji"));
+        xmlWriter->writeDefaultNamespace(toString65(ns_muji));
 
         if (d->isPreparingMujiSession) {
-            xmlWriter->writeEmptyElement(QStringLiteral("preparing"));
+            xmlWriter->writeEmptyElement(u"preparing"_s);
         }
 
         for (const auto &mujiContent : d->mujiContents) {
@@ -618,21 +586,21 @@ void QXmppPresence::toXml(QXmlStreamWriter *xmlWriter) const
 
     // XEP-0319: Last User Interaction in Presence
     if (!d->lastUserInteraction.isNull() && d->lastUserInteraction.isValid()) {
-        xmlWriter->writeStartElement(QStringLiteral("idle"));
-        xmlWriter->writeDefaultNamespace(ns_idle);
-        helperToXmlAddAttribute(xmlWriter, QStringLiteral("since"), QXmppUtils::datetimeToString(d->lastUserInteraction));
+        xmlWriter->writeStartElement(QSL65("idle"));
+        xmlWriter->writeDefaultNamespace(toString65(ns_idle));
+        writeOptionalXmlAttribute(xmlWriter, u"since", QXmppUtils::datetimeToString(d->lastUserInteraction));
         xmlWriter->writeEndElement();
     }
 
     // XEP-0405: Mediated Information eXchange (MIX): Participant Server Requirements
     if (!d->mixUserJid.isEmpty() || !d->mixUserNick.isEmpty()) {
-        xmlWriter->writeStartElement(QStringLiteral("mix"));
-        xmlWriter->writeDefaultNamespace(ns_mix_presence);
+        xmlWriter->writeStartElement(QSL65("mix"));
+        xmlWriter->writeDefaultNamespace(toString65(ns_mix_presence));
         if (!d->mixUserJid.isEmpty()) {
-            helperToXmlAddTextElement(xmlWriter, QStringLiteral("jid"), d->mixUserJid);
+            writeXmlTextElement(xmlWriter, u"jid", d->mixUserJid);
         }
         if (!d->mixUserNick.isEmpty()) {
-            helperToXmlAddTextElement(xmlWriter, QStringLiteral("nick"), d->mixUserNick);
+            writeXmlTextElement(xmlWriter, u"nick", d->mixUserNick);
         }
         xmlWriter->writeEndElement();
     }
